@@ -1,6 +1,8 @@
 #ifndef E_MOD_GADMAN_H
 #define E_MOD_GADMAN_H
 
+#include "e.h"
+
 #define DEFAULT_POS_X  0.1
 #define DEFAULT_POS_Y  0.1
 #define DEFAULT_SIZE_W 0.07
@@ -47,10 +49,18 @@ struct _Manager
 {
    Eina_List   *gadcons[GADMAN_LAYER_COUNT];
    E_Gadcon    *gc_top;
+   E_Gadcon_Location *location[GADMAN_LAYER_COUNT];
    Eina_List   *gadgets[GADMAN_LAYER_COUNT];
+   Ecore_Timer *gadman_reset_timer;
    Evas_Object *movers[GADMAN_LAYER_COUNT];
    Evas_Object *full_bg;
    const char  *icon_name;
+   E_Gadcon_Client *drag_gcc[GADMAN_LAYER_COUNT];
+
+   Eina_List *drag_handlers;
+
+   Eina_List *waiting;
+   Ecore_Event_Handler *add;
    
    int             visible;
    int             use_composite;
@@ -63,6 +73,7 @@ struct _Manager
    E_Module                *module;
    E_Config_Dialog         *config_dialog;
    E_Int_Menu_Augmentation *maug;
+   E_Menu_Category_Callback *mcat;
    E_Action                *action;
 
    E_Config_DD    *conf_edd;
@@ -73,13 +84,23 @@ extern Manager *Man;
 
 void             gadman_init(E_Module *m);
 void             gadman_shutdown(void);
-E_Gadcon_Client *gadman_gadget_add(const E_Gadcon_Client_Class *cc, Gadman_Layer_Type layer);
-void             gadman_gadget_del(E_Gadcon_Client *gcc);
-E_Gadcon_Client *gadman_gadget_place(E_Config_Gadcon_Client *cf, Gadman_Layer_Type layer, E_Zone *zone);
+E_Gadcon_Client *gadman_gadget_add(const E_Gadcon_Client_Class *cc, E_Gadcon_Client *, Gadman_Layer_Type layer);
 void             gadman_gadget_edit_start(E_Gadcon_Client *gcc);
 void             gadman_gadget_edit_end(void *data, Evas_Object *obj, const char *emission, const char *source);
 void             gadman_gadgets_toggle(void);
 void             gadman_update_bg(void);
-E_Gadcon        *gadman_gadcon_get(const E_Zone *zone, Gadman_Layer_Type layer);
+Eina_Bool gadman_gadget_add_handler(void *d, int type, E_Event_Gadcon_Client_Add *ev);
 
+E_Config_Dialog *_config_gadman_module(E_Container *con, const char *params __UNUSED__);
+
+/**
+ * @addtogroup Optional_Gadgets
+ * @{
+ *
+ * @defgroup Module_Gadman Gadman (Gadget Manager)
+ *
+ * Manages @ref Optional_Gadgets on desktop or overlay.
+ *
+ * @}
+ */
 #endif
