@@ -1,6 +1,3 @@
-/*
- * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
- */
 #ifndef E_MOD_MAIN_H
 #define E_MOD_MAIN_H
 
@@ -12,7 +9,6 @@ struct _Config
    /* just config state */
    E_Module        *module;
    Eina_List       *instances;
-   E_Menu          *menu;
    Eina_List       *handlers;
    Eina_List       *items;
    Eina_List	   *config_dialog;
@@ -25,6 +21,8 @@ struct _Config_Item
    int show_zone;
    int show_desk;
    int icon_label;
+   int expand_on_desktop;
+   E_Gadcon_Client *gcc;
 };
 
 EAPI extern E_Module_Api e_modapi;
@@ -37,4 +35,15 @@ void _ibox_config_update(Config_Item *ci);
 void _config_ibox_module(Config_Item *ci);
 extern Config *ibox_config;
 
+/**
+ * @addtogroup Optional_Gadgets
+ * @{
+ *
+ * @defgroup Module_IBox IBox (Box of Iconified Windows)
+ *
+ * Keeps iconified (minimized) windows in a box, showing their icon
+ * and title.
+ *
+ * @}
+ */
 #endif
